from OnionSVG.__init__ import OnionSVG
import argparse

# todo: possible to add a setup option to install as an executable in $PATH?


def str2bool(value):
    if value.lower() in ('yes', 'y', 'true', 't', '1'):
        return True
    elif value.lower in ('no', 'n', 'false', 'f', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError(f"Boolean value expected; received {value}")


parser = argparse.ArgumentParser()
parser.add_argument('path', default = '', help = 'Path to .svg file')
parser.add_argument(
    'peel', nargs = '*', default = 'all',
    help = "Layers to render. \n 'all'/None -> all layers, ['re', '*pattern*']/"
           "['*pattern*'] -> match regular expression")
parser.add_argument('--dpi', default = 200, type = int, help = 'DPI of .svg render')
parser.add_argument('--to', default = None, help = 'Output folder')
parser.add_argument('--list', default = False, const = True, type = str2bool, nargs = '?',
                    help = 'List all layers in the file (does not peel the file!).'
                    )

if __name__ == '__main__':
    args = parser.parse_args()

    svg = OnionSVG(args.path, int(float(args.dpi)), args.to)

    if args.list:
        svg.list()
    else:
        if args.peel is None:
            svg.peel('all', to = args.to, dpi = args.dpi)
        elif args.peel[0] == 're':
            svg.peel(args.peel[1], to = args.to, dpi = args.dpi)
        elif len(args.peel) == 1 and args.peel is not 'all':
            svg.peel(args.peel[0], to = args.to, dpi = args.dpi)
        elif args.peel == 'all':
            svg.peel('all', to = args.to, dpi = args.dpi)
        else:
            argparse.ArgumentTypeError(f"Expected value: ['re', *pattern*], [*pattern*], ['all'] or []")



from lxml import etree
import cairosvg as csvg
import os
import re


"""
Getting cairo to work on windows: 
    https://github.com/preshing/cairo-windows

    --> add cairo.dll to C:\Windows\System32    
"""


def subdir(folder, filename):
    return os.path.join(os.path.join(os.getcwd(), folder), filename)


def check_svg(path):
    name, ext = os.path.splitext(path)
    if ext != 'svg':
        if os.path.exists(name + '.svg'):
            path = name + '.svg'

    try:
        with open(path, 'r') as f:
            data = f.read()
            data = bytes(data, encoding='UTF-8')
    except TypeError:
        raise TypeError(f"Invalid file")

    try:
        root = etree.fromstring(data)
    except etree.XMLSyntaxError:
        raise TypeError(f"Invalid file")

class OnionSVG:
    def __init__(self, path: str, dpi: int = 200, to: str = None):
        self.path = path
        self.dpi = dpi
        self.to = to

        name, ext = os.path.splitext(path)
        if ext != 'svg':
            if os.path.exists(name + '.svg'):
                path = name + '.svg'

        try:
            with open(path, 'r') as f:
                data = f.read()
                data = bytes(data, encoding = 'UTF-8')
        except TypeError:
            raise TypeError(f"Invalid file")

        self.original = data
        ddata = data.decode(encoding = 'UTF-8')
        self.header = bytes(ddata.split('<svg')[0], encoding = 'UTF-8')

        self.root = etree.fromstring(data)

        self.layers = []
        self.get_layers()

    def peel(self, select, dpi = None, to = None):
        print(f"\n Peeling... ({dpi} dpi)\n")

        if to is None:
            self.to = self.path.split('.')[0] + "_layers"
        else:
            self.to = to

        if dpi is None:
            dpi = self.dpi

        if os.path.exists(self.to):
            for file in os.listdir(self.to):
                os.remove(os.path.join(self.to, file))
        else:
            os.mkdir(self.to)

        if select == 'all':
            for layer in self.layers:
                if not layer.constant:
                    for hidden in self.layers:
                        hidden.hide()

                    layer.show()
                    self.save(subdir(self.to, layer.label + '.png'), dpi)
        else:
            pattern = re.compile(select)

            for layer in self.layers:
                if pattern.match(layer.label):
                    if not layer.constant:
                        for hidden in self.layers:
                            hidden.hide()
                        layer.show()

                        self.save(subdir(self.to, layer.label + '.png'), dpi)

        print("\n Done.")

    def get_layers(self):
        for child in self.root.getchildren():
            if child.tag[-1] == 'g':  # child is a layer-level group # todo: find a nicer way to do this
                for attribute in child.attrib.keys():
                    try:
                        if attribute.split('}')[1] == 'label':
                            self.layers.append(Layer(child, child.attrib[attribute]))
                    except IndexError:
                        pass

    def tostring(self):
        return self.header + etree.tostring(self.root)

    def list(self):
        print('\t')
        for layer in self.layers:
            print(f"{layer.label}")

    def save(self, path, dpi):
        csvg.svg2png(self.tostring(), write_to = path, scale = self.dpi2scale(dpi), unsafe = True)

        print(f"{os.path.relpath(path,self.path)}")

    @staticmethod
    def dpi2scale(dpi):
        # todo: this is a bit hacky
        default_dpi = 96
        return dpi/default_dpi


class Layer:
    def __init__(self, root, label):
        self.root = root
        self.label = label

        if self.label[0] == "_":
            self.constant = True
        else:
            self.constant = False

    def __repr__(self):
        try:
            if self.root.attrib['style'] == 'display:none':
                return f"<{self.label}: Hidden>"
            else:
                return f"<{self.label}: Shown>"
        except KeyError:
            return f"<{self.label}: Shown>"

    def hide(self):
        if not self.constant:
            self.root.attrib['style'] = 'display:none'
        else:
            self.show(False)

    def show(self, log_to_console = True):
        self.root.attrib['style'] = 'display:inline'
        if log_to_console:
            print(f"{self.label}", end = " --> ")



#!/usr/bin/env python

from distutils.core import setup
import setuptools

setup(
    name = 'OnionSVG',
    version = 'v0.1.2',
    description = 'Peel your SVG files with Python',
    author = 'ybnd',
    author_email = 'ybnd@tuta.io',
    url = 'https://github.com/ybnd/OnionSVG',
    download_url = 'https://github.com/ybnd/OnionSVG/archive/v0.1.2.tar.gz',
    install_requires = ['lxml', 'cairosvg'],
    packages = ['OnionSVG']
)

